/* PPSWrapper

 * Copyright (C) 2009- Luo Jinghua <sunmoon1997@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more
 */
#ifndef PPSWRAPPER_H
#define PPSWRAPPER_H

#include <stdint.h>
#include <unistd.h>
#include <pthread.h>

#define PPSW_PORT 11994

typedef enum {
    PPSW_REPLY    = 0,
    PPSW_REPLY_ITEM_INFO,
    PPSW_REPLY_READ,
    PPSW_REPLY_EVENT,
    PPSW_REPLY_TIME,
    PPSW_REPLY_DIR,
    PPSW_REPLY_GET_CAT,
    PPSW_REPLY_GET_CHANNEL,
    PPSW_REPLY_EXPIRE_TIME,
    PPSW_HELLO,
    PPSW_CLOSE,
    PPSW_CREATE,
    PPSW_DESTROY,
    PPSW_ADD_ITEM,
    PPSW_REMOVE_ITEM,
    PPSW_PLAY_ITEM,
    PPSW_STOP_ITEM,
    PPSW_READ,
    PPSW_DROP_READ,
    PPSW_GET_FILE_LENGTH,
    PPSW_GET_ITEM_INFO,
    PPSW_SEEK,
    PPSW_GET_EVENT,
    PPSW_GET_TIME,
    PPSW_SAVE_CONFIG_DIR,
    PPSW_GET_CONFIG_DIR,
    PPSW_DEFAULT_CONFIG_DIR,
    PPSW_GET_ACCOUNT_TYPE,
    PPSW_GET_VIP_EXPIRE_TIME,
    PPSW_CHARGE,
    PPSW_VODLIST_INIT,
    PPSW_VODLIST_UNINIT,
    PPSW_VODLIST_GET_CAT,
    PPSW_VODLIST_GET_CHANNEL
} ppsw_msg_id_t;

#define PPSW_SYNC 0x47
#define PPSW_EAGAIN 1

typedef struct {
    /*** sync ***/
    uint8_t    sync;
    /*** message id */
    uint16_t   id;
    /*** sequence number */
    uint16_t   seq;
    /*** payload length */
    uint32_t   len;
} ppsw_msg_t;

#define PPSW_MSG_HEADER_LEN (2 * 2 + 4 + 1)

typedef struct {
    uint32_t  len;
    uint32_t  offset;
    uint8_t  *data;

    uint8_t   local[64];
} ppsw_packet_t;

typedef struct {
    ppsw_msg_t base;
    uint32_t   retval;
    uint8_t*   extra;
} ppsw_reply_msg_t;

typedef struct {
    ppsw_msg_t base;
    char*      oem_name;
    char*      termainal_type;
    char*      device_id;
} ppsw_create_msg_t;

typedef struct {
    ppsw_msg_t base;
    char*      url;
    uint32_t   flags;
} ppsw_add_item_msg_t;

typedef struct {
    ppsw_msg_t base;
    uint32_t   fd;
} ppsw_remove_item_msg_t;

typedef struct {
    ppsw_msg_t base;
    uint32_t   fd;
    char*      valid_code;
} ppsw_play_item_msg_t;

typedef struct {
    ppsw_msg_t base;
    uint32_t   fd;
} ppsw_stop_item_msg_t;

typedef struct {
    ppsw_msg_t base;
    uint32_t   fd;
    uint32_t   length;
    uint32_t   position;
    uint32_t   wait_time_sec;
    uint32_t   wait_time_usec;
} ppsw_read_msg_t;

typedef struct {
    ppsw_msg_t base;
    uint32_t   fd;
} ppsw_drop_read_msg_t;

typedef struct {
    ppsw_msg_t base;
    uint32_t   fd;
    uint32_t   position;
} ppsw_seek_msg_t;

typedef struct {
    int              sock;

    volatile uint16_t seq;
} ppsw_t;

typedef struct {
    ppsw_t          base;

    pid_t           pid;
    pps_callback    callback;
    char*           configdir;
    char            expire_time[1024];

    ppscategory*    categories;
    ppschannel*     channel;

    pthread_mutex_t mutex;

    int             has_thread;
    int             evt_quit;
    pthread_t       evt_thread;
} ppsw_client_t;

#define PPSW_CLIENT_SYNC(super, statment)		\
    do {						\
	ppsw_client_t* self = (ppsw_client_t*)super;	\
							\
	pthread_mutex_lock(&self->mutex);		\
							\
	statment;					\
							\
	pthread_mutex_unlock(&self->mutex);		\
    } while (0)

#define PPSW_EVTBUF_SZ 256
typedef struct ppsw_event_buf {
    int             index;
    pps_event       events[PPSW_EVTBUF_SZ + 1];
    int             rd_pos;
    int             wr_pos;

    struct ppsw_event_buf* next;
} ppsw_event_buf_t;

typedef struct {
    ppsw_t       base;
    int          srv_sock;
    int          done;

    pthread_mutex_t mutex;
    pthread_mutex_t bufmutex;
    ppsw_event_buf_t * bufs;
} ppsw_server_t;

ppsw_t*
ppsw_client_create(int port);

void
ppsw_client_destroy(ppsw_t* super);

ppsw_t*
ppsw_server_create(int fd, int port);

void
ppsw_server_destroy(ppsw_t* super);

void
ppsw_server_run(ppsw_t* super, int infinit);

size_t
ppsw_strlen(const char* s);

int
ppsw_client_send_create(ppsw_t *self,
			const char *oem_name,
			const char * terminal_type,
			const char *device_id,
			pps_callback pcallback);

int
ppsw_client_send_destroy(ppsw_t* self);

int ppsw_client_send_close(ppsw_t* super);

int ppsw_client_send_add_item(ppsw_t* super,
			      const char* url,
			      int flags);

int ppsw_client_send_remove_item(ppsw_t* super,
				 int fd);

int ppsw_client_send_play_item(ppsw_t* super,
			       int fd,
			      const char* valid_code);

int ppsw_client_send_stop_item(ppsw_t* super,
			       int fd);

int ppsw_client_send_get_file_length(ppsw_t* super,
				     int fd);

int ppsw_client_send_seek(ppsw_t* super, int fd,
			  unsigned int position);

ppsvod_iteminfo *ppsw_client_send_get_item_info(ppsw_t* super,
						int fd);

int ppsw_client_send_read(ppsw_t* super, int fd, char *buffer,
			  unsigned int length, unsigned int position,
			  struct timeval *wait_time);

int ppsw_client_send_drop_read(ppsw_t* super,
			       int fd);

int ppsw_client_send_get_event(ppsw_t* super,
			       int* index, pps_event* event);

int ppsw_client_send_get_time(ppsw_t* super,
			      struct timeval* tv);

char* ppsw_client_send_get_config_dir(ppsw_t* super);

int ppsw_client_send_default_config_dir(ppsw_t* super);

int ppsw_client_send_save_config_dir(ppsw_t* super,
				     const char* path);

int ppsw_client_send_vodlist_init(ppsw_t* super);

int ppsw_client_send_vodlist_uninit(ppsw_t* super);

ppscategory* ppsw_client_send_vodlist_getcat(ppsw_t* super, int *catnum);

ppschannel* ppsw_client_send_vodlist_getchannel(ppsw_t* super, int catid, int subcatid,
						int page_size, int page_num);

int ppsw_client_send_get_account_type(ppsw_t* super);

int ppsw_client_send_get_vip_expire_time(ppsw_t* super, char* time);

int ppsw_client_send_charge(ppsw_t* super, const char* password);

int
ppsw_write_item_info(ppsw_packet_t* self,
		     ppsvod_iteminfo* info);

int
ppsw_read_item_info(ppsw_packet_t* p,
		    ppsvod_iteminfo* info);

int
ppsw_server_listen(ppsw_t* super,
		   int port);

int
ppsw_read_reply_msg(ppsw_t* self,
		    ppsw_reply_msg_t* msg);

int
ppsw_send_reply_msg(ppsw_t* self,
		    ppsw_reply_msg_t* msg);

int
ppsw_packet_readl(ppsw_packet_t* self,
		  uint32_t* retval);

int
ppsw_packet_writel(ppsw_packet_t* self,
		   uint32_t retval);

int
ppsw_packet_reads(ppsw_packet_t* self,
		  uint16_t* retval);

int
ppsw_packet_writes(ppsw_packet_t* self,
		   uint16_t retval);

int
ppsw_packet_readb(ppsw_packet_t* self,
		  uint8_t* retval);

int
ppsw_packet_writeb(ppsw_packet_t* self,
		   uint8_t retval);

int
ppsw_packet_read(ppsw_packet_t* self,
		 uint8_t* retval,
		 uint32_t len);

int
ppsw_packet_write(ppsw_packet_t* self,
		  const uint8_t* val,
		  uint32_t len);

int
ppsw_packet_read_str(ppsw_packet_t* self,
		     char** retval,
		     uint16_t* retlen);

int
ppsw_packet_write_str(ppsw_packet_t* self,
		      const char* retval,
		      int len);

int
ppsw_packet_init(ppsw_packet_t* p,
		 uint16_t id,
		 uint32_t len);

void
ppsw_packet_fini(ppsw_packet_t* p);

int
ppsw_packet_reinit(ppsw_packet_t* p,
		   uint16_t id,
		   uint32_t len);
int
ppsw_send_packet(ppsw_t* self,
		 ppsw_packet_t* p);

int
ppsw_reply_packet(ppsw_t* self,
		  ppsw_packet_t* p);

int
ppsw_recv_packet(ppsw_t* self,
		 ppsw_msg_t* msg,
		 ppsw_packet_t* p);

#endif
